//-----------------------------------------------------------------------
// <copyright file="DeviceSpecific.cpp" company="Microsoft">
//      Copyright (c) 2006 Microsoft Corporation. All rights reserved.
// </copyright>
//
// Module:
//      DeviceSpecific.cpp
//
// Description:
//      Implements the device specific code for the driver.
//
//      This file contains device specific information, for example, how
//      to send data to the XYZ device as opposed to the ABC device.
//
//      This sample implementation doesn't do anything - it only contains
//      the stubs to be implemented later
//
//-----------------------------------------------------------------------

#include <windows.h>
#include "DeviceSpecific.h"
#include "StdAfx.h"
#include "Device.h"
#include "Renderer.h"
#include "RenderedData.h"
#include "DataManager.h"
#include "OptimusMiniHelper.h"

COptimusMiniHelper g_OM;

extern bool g_fDeviceIsValid;

class COM3EventHandler : public CEventHandler
{
	virtual void OnKeyDown( int nKey )
	{
		static int dwLastKeyDownCounter = 0;
		static int dwLastKeyDown[3] = {0};
		static DWORD dwLastKeyDownTime[3] = {0};

		// filter errnous keystrokes (when device sends a checksum as a keydown). 
		// normally device sends more then one keycode in a row.
		// thus we skipping "first" keydown and react only on second one.
		
		bool bValid = dwLastKeyDown[nKey-1] + 4 > dwLastKeyDownCounter;

		dwLastKeyDown[nKey-1] = dwLastKeyDownCounter++;

		if ( !bValid )
		{
			return;
		}

		// filter multiply keystrokes
		if( ::GetTickCount() - dwLastKeyDownTime[nKey-1] < 500 )
		{
			return;
		}

		dwLastKeyDownTime[nKey-1] = ::GetTickCount();

		extern CDataManager* g_pDataManager;
		g_pDataManager->SendApplicationEvent(nKey);
	}

	virtual void OnDeviceStateChanged( BOOL p_bState )
	{
	}

}g_EventHandler;

HRESULT DeviceSpecificDisplayBitmap(const CRenderedData RenderedData)
{
    // Send a bitmap to your device here.
    // The bitmap is in buffer RenderedData.pbBitmapData of size (in bytes) RenderedData.cbBitmapData
    // For Example: SendBitmap(RenderedData.pbBitmapData, RenderedData.cbBitmapData);
    (RenderedData);

	if (RenderedData.pbBitmapData)	
	{
		g_OM.ShowPicture(RenderedData.OptimusButton, RenderedData.pbBitmapData);
	}
	else
	{
		BYTE image[96*96*3];
		::memset(image, 0, sizeof(image));

		g_OM.ShowPicture(RenderedData.OptimusButton, image);
	}


    return S_OK;
}


HRESULT DeviceDisplayInitialization(void)
{
    // Optionally check if the device has already been initialized
    // If it has, don't initialize a second time
    // Make sure to set this variable to true after initializing.
    if (true == g_fDeviceIsValid)
    {
        return E_FAIL;
    }

    // Initialize your device here.
	g_OM.Init();

	while( !g_OM.m_bDeviceAlive )
	{
		Sleep(10);
	}

	g_OM.RegisterEventHandler(&g_EventHandler);

	//  .
	g_OM.SendCommand( 2,0 );
	g_OM.SendCommand( 2,0 );

	//   20,40,60%.
	g_OM.SendCommand( 9, 60 );

	g_fDeviceIsValid = true;

    // Set the device capabilities
    CDevice::SetDeviceCaps(L"Optimus mini three", // wszDeviceFriendlyName
                           L"Art. Lebedev Studio", // wszDeviceManufacturer
                           L"1.0.3.1", // wszDeviceFirmwareVersion
                           CDevice::DeviceTypeBitmap, // deviceType
                           96,//320, // dwDeviceHorizontalResolution
                           96,//240, // dwDeviceVerticalResolution
						   CDevice::ColorModeColor, //CDevice::ColorModeMonochrome, // deviceColorMode
                           24); // dwDeviceBitDepth

    // Set the renderer capabilities
	CRendererBase::SetRendererCaps(L"Optimus mini", // wszDefaultBackgroundTitle
                                   L"SideShow Driver\nEnable gadgets in the control panel", // wszDefaultBackgroundBody
                                   L"Tahoma", // wszFontName
                                   12, // wFontSizeInPixels
                                   16, // wLineHeightInPixels
                                   7, // wNumberOfLinesDisplayedOnscreen
                                   13, // wScrollAmountInPixels
                                   3, // wTextOffsetFromLeft
                                   3, // wTextOffsetFromRight
                                   3, // wTextOffsetFromTop
                                   3); // wTextOffsetFromBottom

    return S_OK;
}


HRESULT DeviceButtonsInitialization(void)
{
    // Initialize button callbacks here.
    // This might be a thread, or registering a callback function.
    // Start Read Thread

    return S_OK;
}


HRESULT DeviceDisplayShutdown(void)
{
    // Optionally check if the device has already been shutdown
    // If it has, don't shutdown a second time
    // Make sure to set this variable to false after shutting down.
    if (false == g_fDeviceIsValid)
    {
        return E_FAIL;
    }

    // Shutdown the device here.
	g_OM.Close();
	g_fDeviceIsValid = false;

    CRendererBase::ReleaseRendererCaps();
    CDevice::ReleaseDeviceCaps();

    return S_OK;
}


HRESULT DeviceButtonsShutdown(void)
{
    // Shutdown button callbacks here.
    // This might be killing the thread or unregistering the callback

    return S_OK;
}

