#include "StdAfx.h"
#include "OptimusMiniHelper.h"

COptimusMiniHelper::COptimusMiniHelper()
{
	m_lpEventHandler = NULL;

	for (int i=0;i<MAX_EVENT;i++)
	{
		m_hEvents[i] = ::CreateEvent( NULL, FALSE, FALSE, NULL );
	}

	m_nLastShownPicture = 0;

#ifdef PORT_OVERLAPPED
	memset( &m_objOverlapped, 0, sizeof(m_objOverlapped) );
	if ( PORT_CAN_USE_OVERLAPPED )
 	{
		m_objOverlapped.hEvent = m_hEvents[OVERLAP_EVENT];
	}
#endif

#ifdef FEATURE_LAYOUT
	m_bChangeLayout = FALSE;
#endif 

#ifdef FEATURE_REPAINT
	m_bRepaintPictures = FALSE;
#endif 

	::InitializeCriticalSection( &m_csPictureLocker );

	m_hDeviceThread = ::CreateThread( NULL, 0, DeviceThreadProc, this, 0, &m_dwDeviceThreadId );

	for (int i=0; i<3; i++)
	{
		m_lpPictures[i].Init();
	}
}

COptimusMiniHelper::~COptimusMiniHelper(void)
{
	Close();

	::TerminateThread( m_hDeviceThread, 0 );

	::DeleteCriticalSection( &m_csPictureLocker );

	for (int i=0;i<MAX_EVENT;i++)
	{
		::CloseHandle( m_hEvents[i] );
	}
}

void COptimusMiniHelper::RegisterEventHandler( CEventHandler* p_lpEventHandler )
{ 
	m_lpEventHandler = p_lpEventHandler;
}

/* *************************************************** client thread *************************************************** */
void COptimusMiniHelper::ComparePictures( LPBYTE p_lpPictureA, LPBYTE p_lpPictureB, int* p_lpFirstLine, int* p_lpLastLine )
{
	int nFirstLine=-1, nLastLine=-1;


	for( int nRow=0; nRow<SCREEN_SIZE; nRow++ )
	{
		LPBYTE lpA = p_lpPictureA + nRow*SCREEN_SIZE*3;
		LPBYTE lpB = p_lpPictureB + nRow*SCREEN_SIZE*3;

		for( int nCol=0; nCol<SCREEN_SIZE*3; nCol++, lpA++, lpB++ )
		{
			if( *lpA != *lpB )
			{
				if( nFirstLine == -1 )
				{
					nFirstLine = nRow;
				}
				
				nLastLine = nRow+1;
				break;
			}
		}
	}

	*p_lpFirstLine = nFirstLine;
	*p_lpLastLine = nLastLine;

	DEBUG(DEBUG_PUBLIC_DETAIL,printf( "Images compared %d - %d\n", *p_lpFirstLine, *p_lpLastLine ));
}

int COptimusMiniHelper::ShowPicture( int p_nButton, LPBYTE p_lpPictureBytes, BOOL p_bBlock )
{
	if( p_lpPictureBytes == NULL || p_nButton < 1 || p_nButton > 3 )
	{
		return COMMAND_FAILED;
	}

	DEBUG(DEBUG_PUBLIC,printf( "ShowPictureHelper: %d\n", p_nButton ));

	LPSHOW_PICTURE_COMMAND lpCommand = &m_lpPictures[p_nButton-1];

#ifdef FEATURE_BLOCKING

	if ( p_bBlock && ::GetCurrentThreadId() == m_dwDeviceThreadId )
	{
		return COMMAND_FAILED;
	}

	int nResult=COMMAND_FAILED;
	
	::EnterCriticalSection( &m_csPictureLocker );

	int *lpPreviousResult = lpCommand->m_lpResult;
	HANDLE hPreviousEvent = lpCommand->m_hCompleteEvent;

	HANDLE hCompleteEvent = lpCommand->m_hCompleteEvent = NULL;

	ComparePictures( p_lpPictureBytes, lpCommand->m_bDrawnBytes, &lpCommand->m_nFirstLine, &lpCommand->m_nLastLine );

	if( ! ( lpCommand->m_bSent = lpCommand->m_nFirstLine == -1 ) ) // Picture differs
	{
		::CopyMemory( lpCommand->m_bBytes, p_lpPictureBytes, SCREEN_BUFFER_SIZE );

		if ( p_bBlock )
		{
			lpCommand->m_hCompleteEvent = hCompleteEvent = ::CreateEvent( NULL, FALSE, FALSE, NULL );
			lpCommand->m_lpResult = &nResult;
		}
	}

	::LeaveCriticalSection( &m_csPictureLocker );

	if ( hPreviousEvent != NULL )
	{
		*lpPreviousResult = 0;
		::SetEvent( hPreviousEvent );
	}

	::SetEvent( m_hEvents[COMMAND_EVENT] );

	if ( hCompleteEvent != NULL )
	{
		::WaitForSingleObject( hCompleteEvent, INFINITE );
		::CloseHandle( hCompleteEvent );
		return nResult;
	}

#else // FEATURE_BLOCKING

	::EnterCriticalSection( &m_csPictureLocker );

	ComparePictures( p_lpPictureBytes, lpCommand->m_bDrawnBytes, &lpCommand->m_nFirstLine, &lpCommand->m_nLastLine );

	if( ! ( lpCommand->m_bSent = lpCommand->m_nFirstLine == -1 ) ) // Picture differs
	{
		::CopyMemory( lpCommand->m_bBytes, p_lpPictureBytes, SCREEN_BUFFER_SIZE );
	}

	::LeaveCriticalSection( &m_csPictureLocker );

	::SetEvent( m_hEvents[COMMAND_EVENT] );

#endif // FEATURE_BLOCKING

	return 0;
}

#ifdef FEATURE_LAYOUT

void COptimusMiniHelper::ChangeLayout()
{
	m_bChangeLayout = TRUE;
}

#endif // FEATURE_LAYOUT

#ifdef FEATURE_REPAINT

void COptimusMiniHelper::RepaintPictures()
{
	m_bRepaintPictures = TRUE;
	::SetEvent( m_hEvents[COMMAND_EVENT] );
}

#endif // FEATURE_REPAINT

int COptimusMiniHelper::SendCommand( BYTE c1, BYTE c2, BYTE c3, BYTE c4, BOOL p_bBlock )
{

	DEBUG(DEBUG_PUBLIC,printf( "SendCommandHelper: %d, %d\n", c1, c2 )); 

#ifdef FEATURE_BLOCKING
	if ( p_bBlock && ::GetCurrentThreadId() == m_dwDeviceThreadId )
	{
		return COptimusMini::SendCommand( c1, c2, c3, c4 );
	}

	int nResult=COMMAND_FAILED;
#endif

	OPTIMUS_COMMAND cmd = { 
		c1, c2, c3, c4, 
#ifdef FEATURE_BLOCKING
		p_bBlock ? ::CreateEvent( NULL, FALSE, FALSE, NULL ) : NULL,
		&nResult
#endif
	};

	m_qCommandQueue.push( cmd );
	::SetEvent( m_hEvents[COMMAND_EVENT] );

#ifdef FEATURE_BLOCKING
	if ( p_bBlock )
	{
		::WaitForSingleObject( cmd.m_hCompleteEvent, INFINITE );
		::CloseHandle( cmd.m_hCompleteEvent );
		return nResult;
	}
#endif

	return 0;
}

int COptimusMiniHelper::SendCommand( BYTE c1, BYTE c2, BOOL p_bBlock )
{
	return SendCommand( c1, c2, 0, 0, p_bBlock );
}

BOOL COptimusMiniHelper::Init()
{
	::SetEvent( m_hEvents[0] );
	return FALSE;
}

void COptimusMiniHelper::Close()
{
	::SetEvent( m_hEvents[1] );
	::WaitForSingleObject( m_hEvents[4], INFINITE );
}

/* *************************************************** helper thread *************************************************** */

void COptimusMiniHelper::ShowPictureInternal( int p_nButton )
{

	BOOL bShow = TRUE;

	LPSHOW_PICTURE_COMMAND lpPicture = &m_lpPictures[p_nButton-1];

#ifdef FEATURE_LAYOUT
	if ( m_bChangeLayout )
	{
		for( int i=0; i<3; i++ )
		{
			// 2 - not sent, 1 - sent, not shown, 0 - sent, shown
			m_lpPictures[ i ].m_iLayoutState = 2;
		}
		m_bChangeLayout = FALSE;
	}
#endif 

	::EnterCriticalSection( &m_csPictureLocker );

	SHOW_PICTURE_COMMAND csPicture = *lpPicture;
	csPicture.m_nFirstLine = 0;
	csPicture.m_nLastLine = SCREEN_SIZE; // temporal, should be commented later

	lpPicture->Shown();

#ifdef FEATURE_LAYOUT
	if (lpPicture->m_iLayoutState == 2)
	{
		bShow = FALSE;
		lpPicture->m_iLayoutState = 1;
	}
#endif 
	
	::LeaveCriticalSection( &m_csPictureLocker );

	int nResult=COptimusMini::ShowPicture(p_nButton, csPicture.m_bBytes, csPicture.m_nFirstLine, csPicture.m_nLastLine, bShow );

#ifdef FEATURE_BLOCKING
	if ( csPicture.m_hCompleteEvent != NULL )
	{
		*csPicture.m_lpResult = nResult;
		::SetEvent( csPicture.m_hCompleteEvent );
	}
#endif

#ifdef FEATURE_LAYOUT
	if ( !m_bChangeLayout )
	{
		for (int i=0;i<3;i++)
		{
			if( m_lpPictures[i].m_iLayoutState == 2 && !m_lpPictures[i].m_bSent )
			{
				// another picture for layout ready
				return;
			}
		}

		// no more pictures for layout, showing what is ready
		for (int i=0;i<3;i++)
		{
			if( m_lpPictures[i].m_iLayoutState == 1 )
			{
				COptimusMini::SendCommand( 4, i+1 );
			}
			m_lpPictures[i].m_iLayoutState = 0;
		}
	}
#endif
}

void COptimusMiniHelper::SendCommandInternal()
{
	while( m_qCommandQueue.size() > 0 )
	{
		OPTIMUS_COMMAND cmd = m_qCommandQueue.front();
		m_qCommandQueue.pop();

		int nResult=COptimusMini::SendCommand( cmd.m_c1, cmd.m_c2, cmd.m_c3, cmd.m_c4 );
		
#ifdef FEATURE_BLOCKING
		if ( cmd.m_hCompleteEvent != NULL )
		{
			*cmd.m_lpResult = nResult;
			::SetEvent( cmd.m_hCompleteEvent );
		}
#endif
	}
}

void COptimusMiniHelper::OnKeyDownInternal()
{
	BYTE buf;
	DWORD lRes;

	if( !m_objPort.ReadFile( &buf, 1, &lRes ) )
	{
		ReportProblem( PROBLEM_READ_ERROR );
		return;
	}

	if ( lRes==0 )
	{
		ReportProblem( PROBLEM_READ_TIMEOUT );
		return;
	}

	if( buf>=1 && buf<=3 )
	{
		OnKeyDown( buf );
	}
}

void COptimusMiniHelper::FindPictureInternal()
{

#ifdef FEATURE_REPAINT
	if ( m_bRepaintPictures )
	{
		for( int i=0; i<3; i++ )
		{
			::EnterCriticalSection( &m_csPictureLocker );
			m_lpPictures[ i ].m_bSent = FALSE;
			m_lpPictures[ i ].m_nFirstLine = 0;
			m_lpPictures[ i ].m_nLastLine = SCREEN_SIZE;
			::LeaveCriticalSection( &m_csPictureLocker );
	
			ShowPictureInternal( i+1 );
		}
		m_bRepaintPictures = FALSE;
	} else
#endif
	{
		int nButton = m_nLastShownPicture % 3 +1 ;

		for( int i=0; i<3; i++, nButton= nButton % 3 + 1 )
		{
			if( !m_lpPictures[ nButton-1 ].m_bSent )
			{
				DEBUG(DEBUG_INTERNAL_DETAIL,printf( "Found image %d\n", nButton ));
				break;
			}
		}

		if( m_lpPictures[ nButton-1 ].m_bSent )
		{
			return;
		}

		ShowPictureInternal(nButton);

		m_nLastShownPicture = nButton;
	}
	
	for( int i=0; i<3; i++ )
	{
		if( !m_lpPictures[ i ].m_bSent )
		{
			::SetEvent( m_hEvents[COMMAND_EVENT] );
		}
	}
}

void COptimusMiniHelper::OnKeyDown( BYTE p_cValue )
{
	if( m_lpEventHandler != NULL )
	{
		m_lpEventHandler->OnKeyDown( p_cValue );
	}

}

void COptimusMiniHelper::OnDeviceStateChanged( BOOL p_bState )
{
	if (m_bDeviceAlive != p_bState ){
	
		m_bDeviceAlive = p_bState;

		if( m_lpEventHandler != NULL )
		{
			m_lpEventHandler->OnDeviceStateChanged( p_bState );
		}
	}
}


#ifdef PORT_OVERLAPPED
#define PORT_OVERLAPPED_ACTION(yes,no) m_objOverlapped.hEvent != NULL  ? yes : no
#else
#define PORT_OVERLAPPED_ACTION(yes,no) no
#endif

void COptimusMiniHelper::DeviceThreadProcInternal()
{
	BYTE buf;
	DWORD dwWasRead;

#ifdef FEATURE_PING
	DWORD dwTick = GetTickCount();
#endif

	for(;;)
	{
		if( m_bDeviceAlive )
		{

			PORT_OVERLAPPED_ACTION( NULL, m_objPort.SetTimeout( 100 ) );
			BOOL bRes = m_objPort.ReadFile( &buf, 1, &dwWasRead, PORT_OVERLAPPED_ACTION( &m_objOverlapped, NULL ) );
			PORT_OVERLAPPED_ACTION( NULL, m_objPort.SetTimeout( PORT_TIMEOUT ) );

			if( !bRes )
			{
				ReportProblem( PROBLEM_READ_ERROR );
				continue;
			}

			if( dwWasRead != 0 && buf == 1 )
			{
				OnKeyDownInternal();
			}

		} 

		int res = PORT_OVERLAPPED_ACTION(
			::WaitForMultipleObjects( m_bDeviceAlive ? 4 : 3, m_hEvents, FALSE, 1000 ),
			::WaitForMultipleObjects( 3, m_hEvents, FALSE, m_bDeviceAlive ? 0:100 ) );

		PORT_OVERLAPPED_ACTION( ::CancelIo( m_objPort.m_hHandle ), NULL );

		switch( res )
		{			
			case WAIT_OBJECT_0 + 0: // Init event (higher priority first)
				m_objPort.IsOpened() ? InitDevice() : COptimusMini::Init();
				::ResetEvent( m_hEvents[0] ); // Remove extra calls
				break;

			case WAIT_OBJECT_0 + 1: // Close event
				COptimusMini::Close();
				::SetEvent( m_hEvents[4] );
				break;

			case WAIT_OBJECT_0 + 2: // Execute command event
				SendCommandInternal();
				FindPictureInternal();
				break;
			
#ifdef PORT_OVERLAPPED
			case WAIT_OBJECT_0 + 3: // Keypress detected
				if ( !::GetOverlappedResult( m_objPort.m_hHandle, &m_objOverlapped, &dwWasRead, FALSE ) )
				{
					ReportProblem( PROBLEM_READ_ERROR );
					continue;
				}

				if( dwWasRead != 0 && buf == 1 )
				{
					OnKeyDownInternal();
				}
				break;
#endif
		}

#ifdef FEATURE_PING
		if( m_bDeviceAlive && GetTickCount() - dwTick > 1000 )
		{
#ifdef FEATURE_AUTO_ON
			if (GetTickCount() - dwTick > 8000)	COptimusMini::SendCommand( 2, 0 ); else
#endif
			COptimusMini::SendCommand( 0, 0 );
			dwTick = GetTickCount();
		}
#endif

	}
}

DWORD WINAPI COptimusMiniHelper::DeviceThreadProc( LPVOID lpParam )
{
	DEBUG(DEBUG_NOTIFY,printf( "Device communication thread started...\n" ));
	((COptimusMiniHelper*)lpParam)->DeviceThreadProcInternal();
	return 0;
}
