#include "StdAfx.h"
#include "Port.h"

BOOL CPort::Open( TCHAR* p_szPortName )
{
	if( p_szPortName != NULL )
	{
		int nDigits = 0;
		for( TCHAR* pc=p_szPortName; *pc!='\0'; pc++ )
		{
			if( isdigit( *pc ) )
				nDigits++;
		}
		if( nDigits > 1 )
		{
			_tcscpy( m_szPortName, _T("\\\\.\\") );
		}
		else
		{
			m_szPortName[0] = '\0';
		}

		_tcscat( m_szPortName, p_szPortName );
	}

#ifdef PORT_OVERLAPPED
	memset( &m_objOverlapped, 0, sizeof(m_objOverlapped) );
	if ( PORT_CAN_USE_OVERLAPPED )
	{
		m_objOverlapped.hEvent = ::CreateEvent( 0, FALSE, FALSE, 0 );
	}
#endif

	DEBUG(DEBUG_PORT, printf( "Opening port %s... ", m_szPortName ));

	m_hHandle = CreateFile( m_szPortName, GENERIC_READ|GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 
#ifdef PORT_OVERLAPPED
		m_objOverlapped.hEvent != NULL ? FILE_FLAG_OVERLAPPED | FILE_FLAG_NO_BUFFERING : 
#endif
		FILE_FLAG_NO_BUFFERING, NULL );

    if( m_hHandle == INVALID_HANDLE_VALUE )
	{
		DEBUG(DEBUG_PORT, printf( "failed, gle=%d\n", ::GetLastError() ));
		return FALSE;
	}
	
	DEBUG(DEBUG_PORT, printf( "done\n" ));

	SetDCB();

	return TRUE;
}

void CPort::Close()
{
	if ( m_hHandle != INVALID_HANDLE_VALUE )
	{
		::CloseHandle( m_hHandle );
		m_hHandle = INVALID_HANDLE_VALUE;
	}


#ifdef PORT_OVERLAPPED
	if ( m_objOverlapped.hEvent != NULL )
	{
		::CloseHandle( m_objOverlapped.hEvent );
		m_objOverlapped.hEvent = NULL;
	}
#endif
}

void CPort::SetTimeout( DWORD p_nTimeout )
{
	m_dwTimeout = p_nTimeout;

	COMMTIMEOUTS timeouts = { MAXDWORD, 0, p_nTimeout, 0, PORT_TIMEOUT };
	SetCommTimeouts( m_hHandle,&timeouts );
}

BOOL CPort::WriteFile( LPCVOID lpBuffer, DWORD nNumberOfBytesToWrite, LPDWORD lpNumberOfBytesWritten, LPOVERLAPPED lpOverlapped )
{
#ifdef PORT_OVERLAPPED
	if ( m_objOverlapped.hEvent != NULL )
	{
		BOOL bRes = ::WriteFile( m_hHandle, lpBuffer, nNumberOfBytesToWrite, lpNumberOfBytesWritten, lpOverlapped ? lpOverlapped : &m_objOverlapped );

		if( bRes )
		{
			return TRUE;
		}


		if( ::GetLastError() == ERROR_IO_PENDING )
		{
			if( !lpOverlapped )
			{
				::WaitForSingleObject( m_objOverlapped.hEvent, PORT_TIMEOUT );
				::CancelIo( m_hHandle );
				return ::GetOverlappedResult( m_hHandle, &m_objOverlapped, lpNumberOfBytesWritten, FALSE );
			}

			return TRUE;
		}

		return FALSE;
	}
	else
#endif
	{
		return ::WriteFile( m_hHandle, lpBuffer, nNumberOfBytesToWrite, lpNumberOfBytesWritten, NULL );
	}
}

BOOL CPort::ReadFile( LPVOID lpBuffer, DWORD nNumberOfBytesToRead, LPDWORD lpNumberOfBytesRead, LPOVERLAPPED lpOverlapped )
{
#ifdef PORT_OVERLAPPED
	if ( m_objOverlapped.hEvent != NULL )
	{
		BOOL bRes = ::ReadFile( m_hHandle, lpBuffer, nNumberOfBytesToRead, lpNumberOfBytesRead, lpOverlapped ? lpOverlapped : &m_objOverlapped );

		if( bRes )
		{
			return TRUE;
		}

		if( ::GetLastError() == ERROR_IO_PENDING )
		{
			if( !lpOverlapped )
			{
				::WaitForSingleObject( m_objOverlapped.hEvent, m_dwTimeout );
				::CancelIo( m_hHandle );
				::GetOverlappedResult( m_hHandle, &m_objOverlapped, lpNumberOfBytesRead, FALSE );
			}

			return TRUE;
		}

		return FALSE;
	}
	else 
#endif
	{
		return ::ReadFile( m_hHandle, lpBuffer, nNumberOfBytesToRead, lpNumberOfBytesRead, NULL );
	}
}

BOOL CPort::SetDCB()
{
	DCB dcb;

	if( !GetCommState( m_hHandle ,&dcb ) )
	{
		return FALSE;
	}

	dcb.BaudRate = 1000000;	// for unknown reasons not 1024*1024, works only at this speed
	dcb.ByteSize = 8;		// default 8 in 98, 7 in XP

	//dcb.StopBits=1;		// better not to touch
	//dcb.Parity=0;			// better not to touch

	if( !SetCommState( m_hHandle, &dcb ) )
	{
		return FALSE;
	}

	return TRUE;
}
